/*******************************************************************************
Copyright (c) 2012, Jonathan Hiller

This file is part of the AMF Tools suite. http://amf.wikispaces.com/
AMF Tools is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
AMF Tools is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
See <http://www.opensource.org/licenses/lgpl-3.0.html> for license details.
*******************************************************************************/

#include "MeshSlice.h"
#include <algorithm>
#include "SimpleImage.h"

//#include "nMaterial.h" //TODO: Get rid of this AMF dependence!!!
//#include "AMF_Render.h" //GET rid of this, too...
//#include "AMF_File.h" //...and this...

//for sleep...
#ifdef LINUX
#include <unistd.h>
#endif
#ifdef WIN32
#include <windows.h>
#endif





CMeshSlice::CMeshSlice(void)
{
//	pMaterial = NULL;
//	pObjExt = NULL;
	Initialized = false;
	IsInitializing = false;
	pGetInsideColor = NULL;

}


CMeshSlice::~CMeshSlice(void)
{
}

void CMeshSlice::mySleep(int sleepMs)
{
#ifdef LINUX
    usleep(sleepMs * 1000);   // usleep takes sleep time in us
#endif
#ifdef WIN32
    Sleep(sleepMs);
#endif
}


//bool CMeshSlice::GetLine(std::list<TriEnvelope>* pTriInThisLine, unsigned char* pDataOut, const double Z, const double Y, const double XBegin, const double XEnd, const double XStep) //fills in RGBA data for this line given a pointer to a list of all triangles that could intersect this line.
//{
//
//
//}

void CMeshSlice::MeshChanged(void) //invalidates all cached voxelizing info!
{
	if (Initialized){
		Initialized = false;
		//_TriLayerZ = -1;
		//_TriLayerPZ = -1;
		//CurTriLayer.clear();
		//YEnvMaxIterator = CurTriLayer.begin();
		//_TriLineY = -1;
		//_TriLinePY = -1;
		//CurTriLine.clear();
		//XEnvMaxIterator = CurTriLine.begin();
		//Envelopes.clear();
		//ZEnvMaxIterator = Envelopes.begin(); //but no elements... basically just voids it!
		//_ZActual = -1;
		//_YActual = -1;
		//_XActual = -1;
		//CurTriPoints.clear();
		//XPtIter = CurTriPoints.begin();
		//_TriPointPX = -1;
		//_IsCurInside = false;
		//CurTriPoint.clear();
	}
}
/*
double CMeshSlice::GetClosestFreeZ(double ZIn) //returns the next Z value that does not intersect any vertices
{
	double CurZ = ZIn;
//	double MultFact = 1.0;
	for (std::vector<CVertex>::iterator it = Vertices.begin(); it != Vertices.end(); it++){
		if (it->v.z == CurZ){
			CurZ += 2*CurZ*DBL_EPSILON;
			//MultFact *= 2.0;
			it = Vertices.begin(); //-1 is because it will increment before next time around, and we want to start over from 0
		}
	}
	return CurZ;
}

void CMeshSlice::FillTriLayer(double z, double ZPad) //fills in TriHeight with all triangles that bridge this plane
{
	if (ZPad < 0) ZPad = -ZPad; // only positive!
	if(z == _TriLayerZ && ZPad == _TriLayerPZ) return; //if we've already done this...
	double LastMaxZ = _ZActual + _TriLayerPZ;
	double LastMinZ = _ZActual - _TriLayerPZ;
	_ZActual = GetClosestFreeZ(z);
	double CurMaxZ = _ZActual+ZPad;
	double CurMinZ = _ZActual-ZPad;
	_TriLayerZ = z; //cache these calls
	_TriLayerPZ = ZPad;

		//optimized for sequential ascending Z steps, but works otherwise...
	if (CurMaxZ < LastMaxZ || CurMinZ < LastMinZ){ //if we've gone backwards (downwards) we have to start over from the beginning!
		CurTriLayer.clear(); //start over...
		ZEnvMaxIterator = Envelopes.begin(); 
	}

	//Add in triangles with minimum Z envelopes less than the CurMaxZ
	while (ZEnvMaxIterator != Envelopes.end() && ZEnvMaxIterator->MinZ <= CurMaxZ){ //add in all newly within range, just from where we left the iterator...
		if(ZEnvMaxIterator->MaxZ >= CurMinZ) CurTriLayer.push_back(*ZEnvMaxIterator); // only add if Max Z Envelope is less than CurMinZ before adding
		ZEnvMaxIterator++;
	}

	//remove elements elready in list with max Z envelope less than CurMinZ
	std::list<TriEnvelope>::iterator it = CurTriLayer.begin();
	while (it != CurTriLayer.end()){	//get rid of all elements in the list with maximum values less than MinZ
		if (it->MaxZ<CurMinZ){
			it=CurTriLayer.erase(it); //points to next element after erasing this one
		}
		else it++;
	}

	//sort the CurTriLayer list by y
	CurTriLayer.sort(TriEnvelope::MinYLessThan);
	YEnvMaxIterator = CurTriLayer.begin();
	CurTriLine.clear(); //we've made a new layer array, so clear out all the line info...


}

bool CMeshSlice::FillTriLine(double y, double z, double YPad, double ZPad) //fills in TriHeight with all triangles that bridge this plane
{
	if (YPad < 0) YPad = -YPad; // only positive!
	if(y == _TriLineY && z == _TriLayerZ && YPad == _TriLinePY && ZPad == _TriLayerPZ) return true; //if we've already done this...
	FillTriLayer(z, ZPad); //exits immediately if z is the same as cached...
	if (CurTriLayer.size() == 0){CurTriLine.clear(); return true;} //if no triangles in this layer, no lines! easy!

	double LastMaxY = _YActual + _TriLinePY;
	double LastMinY = _YActual - _TriLinePY;
	_YActual = y; //start here. We will increment if we hit an edge dead on...
	_TriLineY = y; //cache these calls
	_TriLinePY = YPad;

	bool HitEdge = true;
	while(HitEdge){
		HitEdge = false;
		double CurMaxY = _YActual+YPad;
		double CurMinY = _YActual-YPad;
	

		//optimized for sequential ascending Y steps, but works otherwise...
		if (CurMaxY < LastMaxY || CurMinY < LastMinY){ //if we've gone backwards (downwards) we have to start over from the beginning!
			CurTriLine.clear(); //start over...
			YEnvMaxIterator = CurTriLayer.begin(); 
		}

		//Add in triangles with minimum Y envelopes less than the CurMaxY
		while (YEnvMaxIterator != CurTriLayer.end() && YEnvMaxIterator->MinY <= CurMaxY){ //add in all newly within range, just from where we left the iterator...
			if(YEnvMaxIterator->MaxY >= CurMinY) CurTriLine.push_back(*YEnvMaxIterator); // only add if Max Y Envelope is less than CurMinY before adding
			YEnvMaxIterator++;
		}

		//remove elements elready in list with max Y envelope less than CurMinY
		std::list<TriEnvelope>::iterator it = CurTriLine.begin();
		while (it != CurTriLine.end()){	//get rid of all elements in the list with maximum values less than MinY
			if (it->MaxY<CurMinY){
				it=CurTriLine.erase(it); //points to next element after erasing this one
			}
			else it++;
		}

		//sort the CurTriLayer list by y
		CurTriLine.sort(TriEnvelope::MinXLessThan);
		XEnvMaxIterator = CurTriLine.begin();
		CurTriPoint.clear(); //we've made a new line, so clear out all the point info!

		if (!FillTriPoints()){
			_YActual += 2*_YActual*DBL_EPSILON; //increment _YActual and try again. This should make only very small changes to CurTriLine.
			HitEdge = true;
		}

	}
	return true;

}

*/









void CMeshSlice::Initialize()
{
//	if (NumThreads > MAX_SLICE_THREADS) NumThreads = MAX_SLICE_THREADS;
	IsInitializing = true; //in process. don't let other threads jump in on GetSlice
	Initialized = true;

	int NumFacets = Facets.size();
	Envelopes.reserve(NumFacets);
	for (int i=0; i<NumFacets; i++){	
		Envelopes.push_back(TriEnvelope(*this, i));
	}

	std::sort(Envelopes.begin(), Envelopes.end(), TriEnvelope::MinZLessThan); //sort in ascending minimum Z

	for (int i=0; i<MAX_SLICE_THREADS; i++){
		ClearTriLayer(&Envelopes, &CurTriLayers[i], &CurVolumeIters[i]);
		LastZLayer[i] = -DBL_MAX; //set to as low as we go

	}

	Initialized = true;
	IsInitializing = false;
}

bool CMeshSlice::GetSlice(CSimpleImage* pImgOut, double ZHeight, Vec3D SurfDepth, double XMin, double XMax, double YMin, double YMax, int XPix, int YPix, int ThreadNum, std::string* pMessage)
{
	if (!Initialized && !IsInitializing) Initialize(); //should be threadsafe (primitive boolean semaphore)
	else { //assume another thread is initializing. wait around until this object is initialized
		while (IsInitializing) mySleep(50);
	}

	//initial checks...
	if (ThreadNum >= MAX_SLICE_THREADS) return false; //to many threads!
	if (XMax == XMin || YMax == YMin || XPix <= 0 || YPix <= 0) return false;
	if (XMin > XMax){ double tmp = XMin; XMin = XMax; XMax = tmp;} //make sure Max is bigger then Min...
	if (YMin > YMax){ double tmp = YMin; YMin = YMax; YMax = tmp;} //make sure Max is bigger then Min...
	if (SurfDepth.x<0) SurfDepth.x = -SurfDepth.x; //keep all surface depths positive!
	if (SurfDepth.y<0) SurfDepth.y = -SurfDepth.y;
	if (SurfDepth.z<0) SurfDepth.z = -SurfDepth.z;
	double PadPadding = 1.001; //amount to pad the padding so any epsilon changes in our slice location keep all correct triangles within scope
	
	//Elliptical surface extrusion depth: X^2/A^2 + Y^2/B^2 + Z^2/C^2 < 1 means we use closest texture color for this pixel
	//or, B^2*C^2*X^2 + A^2*C^2*Y^2 + A^2*B^2*Z^2 < A^2*B^2*C^2 
	Vec3D EllTexDept = Vec3D(SurfDepth.y*SurfDepth.y*SurfDepth.z*SurfDepth.z, SurfDepth.x*SurfDepth.x*SurfDepth.z*SurfDepth.z, SurfDepth.x*SurfDepth.x*SurfDepth.y*SurfDepth.y); //(B^2*C^2, A^2*C^2, A^2*B^2)
	double EllTexDepthThresh = SurfDepth.x*SurfDepth.x*SurfDepth.y*SurfDepth.y*SurfDepth.z*SurfDepth.z; //A^2*B^2*C^2



//	if (!Initialized) Initialize();
//
	//prepare the image (should already exist...)
	bool IsBw = pImgOut->IsBw();
	if ((!IsBw && !pImgOut->AllocateRgba(XPix, YPix)) || (IsBw && !pImgOut->AllocateBw(XPix, YPix))){
		if (pMessage) *pMessage += "Insufficient memory to create bitmap. Please lower resolution.\n";
		return false;
	}

	//Min and Max sections are the very edge of the picture... but we must sample at the center of pixels!
	Vec3D CurPoint = Vec3D(0,0,ZHeight);
	double XStep = (XMax-XMin)/XPix;
	double YStep = (YMax-YMin)/YPix;
	double XBegin = XMin + XStep/2; //offset half pixel to sample at center of grid points
	double YBegin = YMin + YStep/2; //offset half pixel to sample at center of grid points

	try {
		//prepare this layer of triangles	
		if (ZHeight<LastZLayer[ThreadNum]) ClearTriLayer(&Envelopes, &CurTriLayers[ThreadNum], &CurVolumeIters[ThreadNum]); //if we stepped back in Z, reset the layer buffer and start from the beginning.
		UpdateTriLayer(&Envelopes, ZHeight, PadPadding*SurfDepth.z, &CurTriLayers[ThreadNum], &CurVolumeIters[ThreadNum]); //fill in the list of triangles touching this layer.
		std::list<TriEnvelope*>::iterator CurTriLayerIt = CurTriLayers[ThreadNum].begin(); //Iterator for the layer

		std::list<TriEnvelope*> CurTriLine; //array for all intersection points at the current z height at the current Y value
		std::list<TriEnvelope*>::iterator CurTriLineIt; //Iterator for the Y line

		std::list<TriEnvelope*> CurTriPoint; //array for all intersection points at the current z height at the current Y value at the current X point
		std::list<TriEnvelope*>::iterator CurTriPointIt; //Iterator for the Y line

		std::vector<double> Intersects;
		std::vector<double>::iterator IntersectIt;

		double u, v, dist2, EllDist, MinEllDist, UseU, UseV;
		Vec3D ToSurfPoint;
		CFacet* pUseFacet;

		//do each pixel...
		double CurXLoc, CurYLoc; //, CurXMax, CurXMin;

		int Bpp = pImgOut->GetBpp();

		for (int j=0; j<YPix; j++){
			unsigned char* pCurPixel = pImgOut->GetBits() + (j+1)*Bpp*XPix - Bpp; //start at the LAST pixel on the row (because bitmap coordinates are left-handed)

			CurYLoc = YBegin+YStep*j;
			UpdateTriLine(&CurTriLayers[ThreadNum], CurYLoc, PadPadding*SurfDepth.y, &CurTriLine, &CurTriLayerIt); //fill in the list of triangles touching this line (including padding).
			FillCheckIntersect(&CurTriLine, ZHeight, CurYLoc, &Intersects); //fills in the intersections along this line
			int NumIntersects = Intersects.size();
			if (NumIntersects == 0){
//				pCurPixel += Bpp*XPix;
				continue;
			}

			ClearTriPoint(&CurTriLine, &CurTriPoint, &CurTriLineIt);

			bool IsInside = false; //are we inside or outside the object?
			IntersectIt = Intersects.begin();
			int CurIntersectNum = 0;

			for (int i=0; i<XPix; i++){
				CurXLoc = XBegin+XStep*i;
				double XPad = PadPadding*SurfDepth.x;

				while (CurIntersectNum < NumIntersects && *IntersectIt <= CurXLoc){ //As we advance along x, watch for it we passed 1 (or more) boundary points.
					IsInside = !IsInside;
					CurIntersectNum++;
					IntersectIt++;
				}

				if (IsInside){ //if this point is inside a body, figure out what color to make it.
					bool UseSurfColor = false;
					CurPoint = Vec3D(CurXLoc, CurYLoc, ZHeight);

					if (EllTexDepthThresh != 0){ //if there is a prescribed surface depth
				
						UpdateTriPoint(&CurTriLine, CurXLoc, XPad, &CurTriPoint, &CurTriLineIt);

						//find the UV coords of the closest triangle if any are within the texture depth
						MinEllDist = DBL_MAX;
						for (CurTriPointIt = CurTriPoint.begin(); CurTriPointIt != CurTriPoint.end(); CurTriPointIt++){ //iterate through all the triangle within scope
							CFacet* pCurFacet = &Facets[(*CurTriPointIt)->TriIndex];
							GetTriDist(pCurFacet, &CurPoint, u, v, dist2, ToSurfPoint);

							EllDist = EllTexDept.x*ToSurfPoint.x*ToSurfPoint.x + EllTexDept.y*ToSurfPoint.y*ToSurfPoint.y + EllTexDept.z*ToSurfPoint.z*ToSurfPoint.z; //B^2*C^2*X^2 + A^2*C^2*Y^2 + A^2*B^2*Z^2
							if (EllDist<EllTexDepthThresh && EllDist<MinEllDist){ //if we're closer than the texture depth and haven't found one closer
								MinEllDist = EllDist;
								UseSurfColor = true; //remember that we found
								UseU = u;
								UseV = v;
								pUseFacet = pCurFacet;
							}
						}
					}
					if (UseSurfColor) GetSurfColor(pUseFacet, UseU, UseV, pCurPixel, IsBw);
					else {
						bool GotInterior = GetInteriorColor(&CurPoint, pCurPixel, IsBw, ThreadNum);
						if (!GotInterior) { //default color
							if (IsBw){
								*pCurPixel = 255; //white
								*(pCurPixel+1) = 255; //opaque
							}
							else {
								*pCurPixel = *(pCurPixel+1) = *(pCurPixel+2) = 255; //white
								*(pCurPixel+3) = 255; //opaque
							}
						}
					}
				}
				else {
					if (IsBw) *pCurPixel = *(pCurPixel+1) = 0; 
					else *pCurPixel = *(pCurPixel+1) = *(pCurPixel+2) = *(pCurPixel+3) = 0; //Outside the object. make it transparent
				}

				pCurPixel -= Bpp; //go to next pixel

			}
		}
	}
	catch (std::bad_alloc&){return false;} //keep the program from just dump/crashing.

	LastZLayer[ThreadNum] = ZHeight; //remember what height we are at!
	return true;
}

bool CMeshSlice::FillCheckIntersect(std::list<TriEnvelope*>* pLineIn, double Z, double Y, std::vector<double>* pIntersectOut) //fills in an array of double X values for all intersections with the triangles in the list. Checks for no edges and an even number of intersections.	
{
	static const int MaxTries = 100;
	int NumTries = 0;
	while (!FillIntersect(pLineIn, Z, Y, pIntersectOut) || pIntersectOut->size()%2 == 1){ //while we don't hit any edges (or vertices) and get an even number of hits for the line
		NumTries++;
		if (NumTries > MaxTries) return false;

		if (NumTries%2 == 0) Y *= NumTries*2*DBL_EPSILON; //every other time try incrementing Y, then Z a VERY small amount.
		else Z *= NumTries*2*DBL_EPSILON;

	}
	return true;
}

bool CMeshSlice::FillIntersect(std::list<TriEnvelope*>* pLineIn, const double Z, const double Y, std::vector<double>* pIntersectOut) //fills in an array of double X values for all intersections with the traingles in the list
{
	IntersectionType IntType;
	double TmpXIntersect;

	pIntersectOut->clear();
	if (pLineIn->size() == 0) return true; //shortcut if no triangles near this line

	for (std::list<TriEnvelope*>::iterator it = pLineIn->begin(); it != pLineIn->end(); it++){ //look through each triangle in the list. If the ray is within the envelope add a hit
		if (Y <= (*it)->MaxY && Y >= (*it)->MinY && Z <= (*it)->MaxZ && Z >= (*it)->MinZ){
			IntType = IntersectXRay(&Facets[(*it)->TriIndex], Y, Z, TmpXIntersect);
			if (IntType == IT_EDGE)	return false; //if it hits an edge we'll have to try again
			else if (IntType == IT_INSIDE) pIntersectOut->push_back(TmpXIntersect);
		}
	}

	std::sort(pIntersectOut->begin(), pIntersectOut->end());

	return true;

}


bool CMeshSlice::ClearTriPoint(std::list<TriEnvelope*>* pSortedLineIn, std::list<TriEnvelope*>* pUnsortedPointInOut, std::list<TriEnvelope*>::iterator* pLineCurIterInOut) //Clears the list of triangle envelopes intersecting the point. resets the iterator to the line list. Use if X steps backwards or the layer list changes...
{
	pUnsortedPointInOut->clear(); //reset the triangles within scope...
	*pLineCurIterInOut = pSortedLineIn->begin(); //reset iterator to the beginning of the line of triangles
	return true;
}

bool CMeshSlice::UpdateTriPoint(std::list<TriEnvelope*>* pSortedLineIn, double X, double XPad, std::list<TriEnvelope*>* pUnsortedPointInOut, std::list<TriEnvelope*>::iterator* pLineCurIterInOut) //updates the list of line triEnv intersections. Begins incrementing pLineCurIterInOut in the layer list and adds any triangles whose minimum now lies in the envelope and removes any triangles whose maximum is less than the envelope.
{
	if (XPad < 0) XPad = -XPad; //only positive paddings or else nothing will work.
	if (pSortedLineIn->size() == 0){pUnsortedPointInOut->clear(); return true;} //if no triangles in this layer, no lines! easy!

	double CurMaxX = X+XPad;
	double CurMinX = X-XPad;

	//keep our sliding window of triangles within range of this point current
	//add new triangles with min X envelope less than CurMaxX
	while (*pLineCurIterInOut != pSortedLineIn->end() && (*(*pLineCurIterInOut))->MinX <= CurMaxX){ //add in all newly within range, just from where we left the iterator...
		if((*(*pLineCurIterInOut))->MaxX >= CurMinX) pUnsortedPointInOut->push_back(*(*pLineCurIterInOut)); // only add if Max X Envelope is less than CurMinX before adding
		(*pLineCurIterInOut)++; //increment the iterator either way...
	}

	//remove triangles already in list with max X envelope less than CurMinX (we're fully past them)
	std::list<TriEnvelope*>::iterator it = pUnsortedPointInOut->begin();
	while (it != pUnsortedPointInOut->end()){	//get rid of all elements in the list with maximum values less than MinX
		if ((*it)->MaxX<CurMinX) it=pUnsortedPointInOut->erase(it); //points to next element after erasing this one
		else it++;
	}

	return true;
}

bool CMeshSlice::ClearTriLine(std::list<TriEnvelope*>* pSortedLayerIn, std::list<TriEnvelope*>* pUnsortedLineInOut, std::list<TriEnvelope*>::iterator* pLayerCurIterInOut) //Clears the list in triangle envelopes intersecting the line. resets the iterator to the layer list. Use if Y steps backwards ort the layer list changes...
{
	pUnsortedLineInOut->clear();
	(*pLayerCurIterInOut) = pSortedLayerIn->begin();
	return true;
}

bool CMeshSlice::UpdateTriLine(std::list<TriEnvelope*>* pSortedLayerIn, double Y, double YPad, std::list<TriEnvelope*>* pSortedLineInOut, std::list<TriEnvelope*>::iterator* pLayerCurIterInOut) //updates the list of line triEnv intersections. Begins incrementing pLayerCurIter in the layer list and adds any triangles whose minimum now lies in the envelope and removes any triangles whose maximum is less than the envelope.
{
	if (YPad < 0) YPad = -YPad; //only positive paddings or else nothing will work.
	if (pSortedLayerIn->size() == 0){pSortedLineInOut->clear(); return true;} //if no triangles in this layer, no lines! easy!

	double CurMaxY = Y+YPad;
	double CurMinY = Y-YPad;
	
	//add new triangles with min Y envelope less than CurMaxY
	while (*pLayerCurIterInOut != pSortedLayerIn->end() && (*(*pLayerCurIterInOut))->MinY <= CurMaxY){ //add in all newly within range, just from where we left the iterator...
		if((*(*pLayerCurIterInOut))->MaxY >= CurMinY) pSortedLineInOut->push_back(*(*pLayerCurIterInOut)); // only add if Max Y Envelope is less than CurMinY before adding
		(*pLayerCurIterInOut)++; //increment the iterator either way...
	}

	//remove triangles already in list with max Y envelope less than CurMinY (we're fully past them)
	std::list<TriEnvelope*>::iterator it = pSortedLineInOut->begin();
	while (it != pSortedLineInOut->end()){	//get rid of all elements in the list with maximum values less than MinY
		if ((*it)->MaxY<CurMinY) it=pSortedLineInOut->erase(it); //points to next element after erasing this one
		else it++;
	}

	pSortedLineInOut->sort(TriEnvelope::MinXLessThanP);

	return true;
}

bool CMeshSlice::ClearTriLayer(std::vector<TriEnvelope>* pSortedVolumeIn, std::list<TriEnvelope*>* pSortedLayerInOut, std::vector<TriEnvelope>::iterator* pVolumeCurIterInOut) //Clears the list of triangle envelopes intersecting the layer. resets the iterator to the volume list. Use if Z steps backwards or the volume list changes...
{
	pSortedLayerInOut->clear();
	(*pVolumeCurIterInOut) = pSortedVolumeIn->begin();
	return true;
}

bool CMeshSlice::UpdateTriLayer(const std::vector<TriEnvelope>* pSortedVolumeIn, double Z, double ZPad, std::list<TriEnvelope*>* pSortedLayerInOut, std::vector<TriEnvelope>::iterator* pVolumeCurIterInOut) //updates the list of line triEnv intersections. Begins incrementing pLayerBeginLooking in the layer list and adds any triangles whose minimum now lies in the envelope and removes any triangles whose maximum is less than the envelope.
{
	if (ZPad < 0) ZPad = -ZPad; //only positive paddings or else nothing will work.
	double CurMaxZ = Z+ZPad;
	double CurMinZ = Z-ZPad;

	//Add in triangles with minimum Z envelopes less than the CurMaxZ
	while ((*pVolumeCurIterInOut) != pSortedVolumeIn->end() && (*pVolumeCurIterInOut)->MinZ <= CurMaxZ){ //add in all newly within range, just from where we left the iterator...
		if((*pVolumeCurIterInOut)->MaxZ >= CurMinZ) pSortedLayerInOut->push_back(&(*(*pVolumeCurIterInOut))); // only add if Max Z Envelope is less than CurMinZ before adding
		(*pVolumeCurIterInOut)++;
	}

	//remove elements elready in list with max Z envelope less than CurMinZ
	std::list<TriEnvelope*>::iterator it = pSortedLayerInOut->begin();
	while (it != pSortedLayerInOut->end()){	//get rid of all elements in the list with maximum values less than MinZ
		if ((*it)->MaxZ<CurMinZ){
			it=pSortedLayerInOut->erase(it); //points to next element after erasing this one
		}
		else it++;
	}

	//sort the CurTriLayer list by minimum Y value
	pSortedLayerInOut->sort(TriEnvelope::MinYLessThanP);

	return true;
}

bool CMeshSlice::GetInteriorColor(Vec3D* pCurPoint, unsigned char* pDataOut, bool BwData, int ThreadNum) //returns the color of the object material at this point (assums it is inside the volume)
{
	unsigned char tR, tG, tB, tA;
	if (pGetInsideColor){
		pGetInsideColor(*pCurPoint, &tR, &tG, &tB, &tA, ThreadNum); //, aObjectID);
		if (BwData){
			*pDataOut = (tR+tG+tB)/3; //gray
			*(pDataOut+1) = tA; //alpha
		}
		else {
			*pDataOut = tR;
			*(pDataOut+1) = tG;
			*(pDataOut+2) = tB;
			*(pDataOut+3) = tA;
		}
		return true;
	}
	else return false;

	//double tR, tG, tB, tA;
	//if (pGetInsideColor){
	//	pGetInsideColor(*pCurPoint, &tR, &tG, &tB, &tA, ThreadNum); //, aObjectID);
	//	if (BwData){
	//		*pDataOut = (unsigned char)((tR*255 + tG*255 + tB*255)/3); //gray
	//		*(pDataOut+1) = (unsigned char)(tA*255); //alpha
	//	}
	//	else {
	//		*pDataOut = (unsigned char)(tR*255);
	//		*(pDataOut+1) = (unsigned char)(tG*255);
	//		*(pDataOut+2) = (unsigned char)(tB*255);
	//		*(pDataOut+3) = (unsigned char)(tA*255);
	//	}
	//	return true;
	//}
	//else return false;
}

bool CMeshSlice::GetSurfColor(const CFacet* pFacet, const double UCoord, const double VCoord, unsigned char* pDataOut, bool BwData) //returns the color of this facet at the specified U and V coordinate. Fillis in the 4 bytes past pRgbaOut with the color info
{
	bool UseTexColors = false;
	CTexture* pCurTexture = NULL;
	double PicU, PicV;

	if (pFacet->HasTexture){
		pCurTexture = &Textures[pFacet->Map.TexIndex];
//		CTexture* pCurTexture = NULL;
//		if (pFacet->Map.TexIndex < (int)Textures.size()) pCurTexture = &Textures[pFacet->Map.TexIndex]; //shouldn't need this check here. (if throwing errors then check texmaps before slice!)
							
		//u coord is from vertex 1 to vertex 2, v coord is v1 to v3
		PicU = pFacet->Map.uc[0] + UCoord*(pFacet->Map.uc[1]-pFacet->Map.uc[0]) + VCoord*(pFacet->Map.uc[2]-pFacet->Map.uc[0]);
		PicV = pFacet->Map.vc[0] + UCoord*(pFacet->Map.vc[1]-pFacet->Map.vc[0]) + VCoord*(pFacet->Map.vc[2]-pFacet->Map.vc[0]);
		if (PicU == 1.0) PicU = 1.0-DBL_MIN; //stay off the upper edge without grabbing a pix from 0.0...
		if (PicV == 1.0) PicV = 1.0-DBL_MIN; //stay off the upper edge...

		if (pCurTexture->Tiled){ //if we're tiling the textures do the mod...
			UseTexColors = true;
			PicU = fmod(PicU, 1.0);
			if (PicU < 0) PicU += 1.0; //becuase fmod of a negative number is negative remainder
			PicV = fmod(PicV, 1.0);
			if (PicV < 0) PicV += 1.0; //becuase fmod of a negative number is negative remainder

		}
		else { //not tiling texture, only do texture if we're within [0, 1]
			if (PicU >= 0 && PicU < 1 && PicV >= 0 && PicV < 1) UseTexColors = true;
		}

	}
	if (UseTexColors && pCurTexture){
		int XPix = (int)(PicU*pCurTexture->Width);
		int YPix = (int)(PicV*pCurTexture->Height);
						
		unsigned char* pPix = &(pCurTexture->RGBAImage[4*(YPix*pCurTexture->Width + XPix)]);
		if (BwData){
			*pDataOut = (*pPix + *(pPix+1) + *(pPix+2))/3; //gray
			*(pDataOut+1) = *(pPix+3); //alpha
		}
		else { //rgba
			*pDataOut = *pPix; //Red
			*(pDataOut+1) = *(pPix+1); //Green
			*(pDataOut+2) = *(pPix+2); //Blue
			*(pDataOut+3) = *(pPix+3); //Alpha
		}
	}
	else { //no texture, so linearly interpolate the vertices! (this could be sped up...)
		CColor V0 = Vertices[pFacet->vi[0]].VColor;
		CColor V1 = Vertices[pFacet->vi[1]].VColor;
		CColor V2 = Vertices[pFacet->vi[2]].VColor;

		double R = V0.r + UCoord*(V1.r-V0.r) + VCoord*(V2.r-V0.r);
		double G = V0.g + UCoord*(V1.g-V0.g) + VCoord*(V2.g-V0.g);
		double B = V0.b + UCoord*(V1.b-V0.b) + VCoord*(V2.b-V0.b);
//		*pColor = CColor(R, G, B, 1.0);

		if (BwData){
			*pDataOut = (unsigned char)((R+G+B)/3.0*255); //Grey
			*(pDataOut+1) = 255; //Alpha
		}
		else { //rgba		
			*pDataOut = (unsigned char)(R*255); //Red
			*(pDataOut+1) = (unsigned char)(G*255); //Green
			*(pDataOut+2) = (unsigned char)(B*255); //Blue
			*(pDataOut+3) = 255; //Alpha
		}
	}
	return true;
}









/*
bool CMeshSlice::FillTriPoints(void) //fills points based on _YActual _ZActual, and the tris in TriLine. returns false increments _YActual (to re-do FillTriLine with) and try again.
{
	IntersectionType IntType;
	double TmpXIntersect;

	CurTriPoints.clear();
	for (std::list<TriEnvelope>::iterator it = CurTriLine.begin(); it != CurTriLine.end(); it++){
		if (_YActual <= it->MaxY && _YActual >= it->MinY && _ZActual <= it->MaxZ && _ZActual >= it->MinZ){
			IntType = IntersectXRay(&Facets[it->TriIndex], _YActual, _ZActual, TmpXIntersect);
			if (IntType == IT_EDGE)	return false;
			else if (IntType == IT_INSIDE) CurTriPoints.push_back(TmpXIntersect);
		}
	}

	std::sort(CurTriPoints.begin(), CurTriPoints.end());
	XPtIter = CurTriPoints.begin(); //ready to iterate
	_IsCurInside = false; //not inside the part if we just reset this array!!
	return true;
}

//---------------------------------------------------------------------------
bool CMeshSlice::IsInside(Vec3D* Point, Vec3D TexDepth, CColor* pColor)
//---------------------------------------------------------------------------
{
	if (TexDepth.x<0) TexDepth.x = -TexDepth.x;
	if (TexDepth.y<0) TexDepth.y = -TexDepth.y;
	if (TexDepth.z<0) TexDepth.z = -TexDepth.z;

	FillTriLine(Point->y, Point->z, TexDepth.y, TexDepth.z); //returns very fast if previously used z or y layers...


	//////sliding window of triangles to consider
	double LastMaxX = _XActual + _TriPointPX;
	double LastMinX = _XActual - _TriPointPX;
	_XActual = Point->x;
	_TriPointPX = TexDepth.x;
	double CurMaxX = _XActual + _TriPointPX;
	double CurMinX = _XActual - _TriPointPX;


	//optimized for sequential ascending X steps, but works otherwise...
	if (CurMaxX < LastMaxX || CurMinX < LastMinX){ //if we've gone backwards (downwards) we have to start over from the beginning!
		_IsCurInside = false;
		CurTriPoint.clear();
		XPtIter = CurTriPoints.begin(); //reset the iterators
		XEnvMaxIterator = CurTriLine.begin();
	}

	//Add in triangles with minimum X envelopes less than the CurMaxX
	while (XEnvMaxIterator != CurTriLine.end() && XEnvMaxIterator->MinX <= CurMaxX){ //add in all newly within range, just from where we left the iterator...
		if(XEnvMaxIterator->MaxX >= CurMinX) CurTriPoint.push_back(*XEnvMaxIterator); // only add if Max Z Envelope is less than CurMinZ before adding
		XEnvMaxIterator++;
	}

	//remove elements elready in list with max X envelope less than CurMinZ
	std::list<TriEnvelope>::iterator it = CurTriPoint.begin();
	while (it != CurTriPoint.end()){	//get rid of all elements in the list with maximum values less than MinZ
		if (it->MaxX<CurMinX){
			it=CurTriPoint.erase(it); //points to next element after erasing this one
		}
		else it++;
	}


	//figure out if we're inside or outside the body
	while (XPtIter != CurTriPoints.end() && *XPtIter <= _XActual){ //add in all newly within range, just from where we left the iterator...
		_IsCurInside = !_IsCurInside; //every time we cross a point, we've crossed a boundary!
		XPtIter++;
	}

	//Elliptical surface extrusion depth: X^2/A^2 + Y^2/B^2 + Z^2/C^2 < 1 means we use closest texture color for this pixel
	//or, B^2*C^2*X^2 + A^2*C^2*Y^2 + A^2*B^2*Z^2 < A^2*B^2*C^2 
	Vec3D EllTexDept = Vec3D(TexDepth.y*TexDepth.y*TexDepth.z*TexDepth.z, TexDepth.x*TexDepth.x*TexDepth.z*TexDepth.z, TexDepth.x*TexDepth.x*TexDepth.y*TexDepth.y); //(B^2*C^2, A^2*C^2, A^2*B^2)
	double EllTexDepthThresh = TexDepth.x*TexDepth.x*TexDepth.y*TexDepth.y*TexDepth.z*TexDepth.z; //A^2*B^2*C^2

	if (_IsCurInside){
		if (pColor){
			double u, v, dist2, mindist2 = 9e9, EllDist, MinEllDist = 9e9;
			Vec3D ToSurfPoint;
			*pColor = CColor();

			for (std::list<TriEnvelope>::iterator it = CurTriPoint.begin(); it != CurTriPoint.end(); it++){ //iterate through all the triangle within scope
				CFacet* pCurFacet = &Facets[it->TriIndex];
				GetTriDist(pCurFacet, Point, u, v, dist2, ToSurfPoint);

				EllDist = EllTexDept.x*ToSurfPoint.x*ToSurfPoint.x + EllTexDept.y*ToSurfPoint.y*ToSurfPoint.y + EllTexDept.z*ToSurfPoint.z*ToSurfPoint.z; //B^2*C^2*X^2 + A^2*C^2*Y^2 + A^2*B^2*Z^2
				if (EllDist<EllTexDepthThresh){ //if we're closer than the texture depth...
//				if (dist2<TexDepth*TexDepth){ //if we're closer than the texture depth...
					if (EllDist<MinEllDist){ //FOR NOW just take minimum... eventually average?
//					if (dist2<mindist2){ //FOR NOW just take minimum... eventually average?
						MinEllDist = EllDist;

						double PicU, PicV;
						bool UseTexColors = false;
						CTexture* pCurTexture = NULL;
						if (pCurFacet->HasTexture){	
							if (pCurFacet->Map.TexIndex < (int)Textures.size()) pCurTexture = &Textures[pCurFacet->Map.TexIndex];
							
							//u coord is from vertex 1 to vertex 2, v coord is v1 to v3
							PicU = pCurFacet->Map.uc[0] + u*(pCurFacet->Map.uc[1]-pCurFacet->Map.uc[0]) + v*(pCurFacet->Map.uc[2]-pCurFacet->Map.uc[0]);
							PicV = pCurFacet->Map.vc[0] + u*(pCurFacet->Map.vc[1]-pCurFacet->Map.vc[0]) + v*(pCurFacet->Map.vc[2]-pCurFacet->Map.vc[0]);
							if (PicU == 1.0) PicU = 1.0-DBL_MIN; //stay off the upper edge without grabbing a pix from 0.0...
							if (PicV == 1.0) PicV = 1.0-DBL_MIN; //stay off the upper edge...

							if (pCurTexture && pCurTexture->Tiled){ //if we're tiling the textures do the mod...
								UseTexColors = true;
								PicU = fmod(PicU, 1.0);
								if (PicU < 0) PicU += 1.0; //becuase fmod of a negative number is negative remainder
								PicV = fmod(PicV, 1.0);
								if (PicV < 0) PicV += 1.0; //becuase fmod of a negative number is negative remainder

							}
							else { //not tiling texture, only do texture if we're within [0, 1]
								if (PicU >= 0 && PicU < 1 && PicV >= 0 && PicV < 1) UseTexColors = true;
							}

						}
						if (UseTexColors && pCurTexture){
							int XPix = (int)(PicU*pCurTexture->Width);
							//int YPix = Texture.Height - (int)(PicV*Texture.Height)-1;
							int YPix = (int)(PicV*pCurTexture->Height);
						
							unsigned char* pPix = &(pCurTexture->RGBAImage[4*(YPix*pCurTexture->Width + XPix)]);
							*pColor = CColor(*pPix/255.0, *(pPix+1)/255.0, *(pPix+2)/255.0, *(pPix+3)/255.0);
						}
						else { //no texture, so linearly interpolate the vertices!
							CColor V0 = Vertices[pCurFacet->vi[0]].VColor;
							CColor V1 = Vertices[pCurFacet->vi[1]].VColor;
							CColor V2 = Vertices[pCurFacet->vi[2]].VColor;

							double R = V0.r + u*(V1.r-V0.r) + v*(V2.r-V0.r);
							double G = V0.g + u*(V1.g-V0.g) + v*(V2.g-V0.g);
							double B = V0.b + u*(V1.b-V0.b) + v*(V2.b-V0.b);
							*pColor = CColor(R, G, B, 1.0);
						}
					}
				}
			}


		}
		return true;
	}
	else return false;

}
*/
IntersectionType CMeshSlice::IntersectXRay(CFacet* pFacet, double y, double z, double& XIntersect)
{
	//http://www.blackpawn.com/texts/pointinpoly/default.html

	Vec3D vA = Vertices[pFacet->vi[0]].v;
	Vec3D vB = Vertices[pFacet->vi[1]].v;
	Vec3D vC = Vertices[pFacet->vi[2]].v;

	double v0y = vC.y-vA.y; //u
	double v0z = vC.z-vA.z;
	double v1y = vB.y-vA.y; //v
	double v1z = vB.z-vA.z;
	double v2y = y-vA.y;
	double v2z = z-vA.z;

	double dot00=v0y*v0y+v0z*v0z;
	double dot01=v0y*v1y+v0z*v1z;
	double dot02=v0y*v2y+v0z*v2z;
	double dot11=v1y*v1y+v1z*v1z;
	double dot12=v1y*v2y+v1z*v2z;

	double invDenom = 1.0/(dot00*dot11-dot01*dot01);
	double u=(dot11*dot02-dot01*dot12)*invDenom;
	double v=(dot00*dot12-dot01*dot02)*invDenom;

	if ((u >= 0) && (v >= 0) && (u + v <= 1)){
		if (u == 0 || v == 0 || u+v==1) 
			return IT_EDGE;
		else {
			XIntersect = vA.x+u*(vC.x-vA.x)+v*(vB.x-vA.x);
			return IT_INSIDE;
		}
	}
	else return IT_OUTSIDE;
}

bool CMeshSlice::GetTriDist(CFacet* pFacetCheck, Vec3D* pPointIn, double& UOut, double& VOut, double& Dist2Out, Vec3D& ToSurfPointOut) //gets distance of provided point to closest UV coordinate of within the triangle. returns true if sensible distance, otherwise false
{
	//http://www.geometrictools.com/Documentation/DistancePoint3Triangle3.pdf

	//Regions:
	//    E1 (t)
	// \ 2|
	//   \|
	//	  |\
	//    |  \    1
	//  3 |    \
	//    | 0    \
	// ___|________\____   E0 (s)
	//    |          \ 6
	// 4  |    5       \

	

	Vec3D B = Vertices[pFacetCheck->vi[0]].v;
	Vec3D E0 = Vertices[pFacetCheck->vi[1]].v - B; //(s)
	Vec3D E1 = Vertices[pFacetCheck->vi[2]].v - B; //(t)
	Vec3D D = B - *pPointIn;

	double a = E0.Dot(E0);
	double b = E0.Dot(E1);
	double c = E1.Dot(E1);
	double d = E0.Dot(D);
	double e = E1.Dot(D);
	double f = D.Dot(D);

	double det=a*c-b*b;
	double s = b*e-c*d;
	double t = b*d-a*e;

	//determine which region...
	//Q(s,t) = as^2+2bst+ct^2+2ds+2et+f
	if (s+t <= det){
		if (s<0){
			if (t<0){ //Region 4 //TODO: CHECK ACCURATE!
				// Grad(Q) = 2(as+bt+d,bs+ct+e) :: (derivative w respect to s, t)
				// (0,1)*Grad(Q(0,0)) = (0,1)*(d,e) = e :: is grad from tip of region 2 in directon of leg toward origin (0,1)...
				// (1,0)*Grad(Q(0,0)) = (1,0)*(d,e) = d
				// min on edge t=0 if (0,1)*Grad(Q(0,0)) < 0 )
				// min on edge s=0 otherwise
				if (e<0){ // minimum on edge t=0
					t=0;
					t=(d >=0 ? 0 :(-d >=a ? 1:-d/a));
				}
				else { // minimum on edge s=0
					s=0;
					t=(e >=0 ? 0 :(-e >=c ? 1:-e/c));
				}
			}
			else { //Region 3
				// F(t) = Q(0,t) = ct^2 + 2et + f
				// F(t)/2 = ct+e
				// F(T) = 0 when T = -e/c
				s=0;
				t=(e >=0 ? 0 :(-e >=c ? 1:-e/c));
			}
		}
		else if (t<0){ //Region 5 (like region 3)
			// F(s) = Q(s,0) = as^2 + 2ds + f
			// F(s)/2 = as+d
			// F(S) = 0 when S = -d/a
			t=0;
			t=(d >=0 ? 0 :(-d >=a ? 1:-d/a));
		}
		else { //Region 0
			double invdet = 1/det;
			s *= invdet;
			t *= invdet;
		}
	}
	else {
		if (s<0){ //Region 2
			// Grad(Q) = 2(as+bt+d,bs+ct+e) :: (derivative w respect to s, t)
			// (0,-1)*Grad(Q(0,1)) = (0,-1)*(b+d,c+e) = -(c+e) :: is grad from tip of region 2 in directon of leg toward origin (0,1)...
			// (1,-1)*Grad(Q(0,1)) = (1,-1)*(b+d,c+e) = (b+d)-(c+e)
			// min on edge s+t=1 if (1,-1)*Grad(Q(0,1)) < 0 )
			// min on edge s=0 otherwise
			double tmp0 = b+d;
			double tmp1 = c+e;
			if ( tmp1 > tmp0 ){ // minimum on edge s+t=1
				double numer = tmp1 - tmp0;
				double denom = a-2*b+c;
				s = ( numer >= denom ? 1 : numer/denom );
				t = 1-s;
			}
			else { // minimum on edge s=0
				s = 0;
				t = ( tmp1 <= 0 ? 1 : ( e >= 0 ? 0 : -e/c ) );
			}
		}
		else if (t<0){ //Region 6 //TODO: CHECK ACCURATE!
			// Grad(Q) = 2(as+bt+d,bs+ct+e) :: (derivative w respect to s, t)
			// (-1,0)*Grad(Q(1,0)) = (-1,0)*(a+d,b+e) = -(a+d) :: is grad from tip of region 2 in directon of leg toward origin (0,1)...
			// (-1,1)*Grad(Q(1,0)) = (-1,1)*(a+d,b+e) = (b+e)-(a+d)
			// min on edge s+t=1 if (-1,1)*Grad(Q(0,1)) < 0 )
			// min on edge t=0 otherwise
			double tmp0 = b+e;
			double tmp1 = a+d;
			if ( tmp1 > tmp0 ){ // minimum on edge s+t=1
				double numer = tmp1 - tmp0;
				double denom = a-2*b+c;
				t = ( numer >= denom ? 1 : numer/denom );
				s = 1-t;
			}
			else { // minimum on edge t=0
				t = 0;
				s = ( tmp1 <= 0 ? 1 : ( d >= 0 ? 0 : -d/a ) );
			}
		}
		else { //Region 1
			// F(s) = Q(s,1-s) = (a-2b+c)s^2 + 2(b-c+d-e)s + (c+2e+f)
			// F(s)/2 = (a-2b+c)s + (b-c+d-e)
			// F(S) = 0 when S = (c+e-b-d)/(a-2b+c)
			// a-2b+c = |E0-E1|^2 > 0, so only sign of c+e-b-d need be considered
			double numer = c+e-b-d;
			if (numer <= 0) s=0;
			else {
				double denom = a-2*b+c;
				s = (numer >= denom ? 1 : numer/denom);
			}
			t=1-s;
		}
	}

	//check one or the other? should be same!
	Vec3D ClosestPoint = B+s*E0+t*E1;
	ToSurfPointOut = ClosestPoint-*pPointIn; //OUTPUT!
//	double D2 = ToSurfPointOut.Length2();
//	double D22 = a*s*s+2*b*s*t+c*t*t+2*d*s+2*e*t+f;

	Dist2Out = a*s*s+2*b*s*t+c*t*t+2*d*s+2*e*t+f;

	UOut = s;
	VOut = t;
//	Dist2Out = D22;
	return true;
}
/*
bool CMeshSlice::GetSlice(CSimpleImage* pImgOut, double ZHeight, Vec3D SurfDepth, double XMin, double XMax, double YMin, double YMax, int XPix, int YPix, int ThreadNum, std::string* pMessage)
{
	//initial checks...
	if (XMax == XMin || YMax == YMin || XPix <= 0 || YPix <= 0) return false;
	if (XMin > XMax){ double tmp = XMin; XMin = XMax; XMax = tmp;} //make sure Max is bigger then Min...
	if (YMin > YMax){ double tmp = YMin; YMin = YMax; YMax = tmp;} //make sure Max is bigger then Min...
	
	if (!Initialized) Initialize();

	//prepare the image (should already exist...)
	if (!pImgOut->AllocateRgba(XPix, YPix)){
		if (pMessage) *pMessage += "Insufficient memory to create bitmap. Please lower resolution.\n";
		return false;
	}

	CColor CurColor;

	//Min and Max sections are the very edge of the picture... but we must sample at the center of pixels!
	Vec3D CurPoint = Vec3D(0,0,ZHeight);
	double XStep = (XMax-XMin)/XPix;
	double YStep = (YMax-YMin)/YPix;
	double XBegin = XMin + XStep/2; //offset half pixel to sample at center of grid points
	double YBegin = YMin + YStep/2; //offset half pixel to sample at center of grid points
	
	double tR, tG, tB, tA;

	//do each pixel...
	//QRgb* pCurPixel;
	unsigned char* pCurPixel = pImgOut->GetRGBABits();
	for (int j=0; j<YPix; j++){
		CurPoint.y = YBegin+YStep*j;
//		pCurPixel = (QRgb*)pImgOut->scanLine(j);
		for (int i=0; i<XPix; i++){
			CurPoint.x = XBegin+XStep*i;
			if (IsInside(&CurPoint, SurfDepth, &CurColor)){
				if (CurColor.a == -1){ //if there is no surface texture... (added from within IsInside())
					if (pGetInsideColor){
						pGetInsideColor(CurPoint, &tR, &tG, &tB, &tA, ThreadNum); //, aObjectID);
						*pCurPixel = (unsigned char)(tR*255);
						*(pCurPixel+1) = (unsigned char)(tG*255);
						*(pCurPixel+2) = (unsigned char)(tB*255);
						*(pCurPixel+3) = (unsigned char)(tA*255);
					}
					else { //default color
						*pCurPixel = *(pCurPixel+1) = *(pCurPixel+2) = 255; //white
						*(pCurPixel+3) = 255;
					}
				}
				else { //if there was a surface texture here, use the color returned by IsInside()
					*pCurPixel = (unsigned char)(CurColor.r*255);
					*(pCurPixel+1) = (unsigned char)(CurColor.g*255);
					*(pCurPixel+2) = (unsigned char)(CurColor.b*255);
					*(pCurPixel+3) = (unsigned char)(CurColor.a*255);
				}
			}
			else { //if not inside, make it transparent!
				*pCurPixel = *(pCurPixel+1) = *(pCurPixel+2) = *(pCurPixel+3) = 0;
			}
			pCurPixel += 4; //go to next pixel
		}
	}
	return true;
}
*/
//void CMeshSlice::Initialize()
//{
//	Initialized = true;
//
//	int NumFacets = Facets.size();
//	Envelopes.reserve(NumFacets);
//	for (int i=0; i<NumFacets; i++){	
//		Envelopes.push_back(TriEnvelope(*this, i));
//	}
//
//	std::sort(Envelopes.begin(), Envelopes.end(), TriEnvelope::MinZLessThan); //sort in ascending minimum Z
//	ZEnvMaxIterator = Envelopes.begin();
//}

